from odoo.addons.somconnexio.tests.helper_service import contract_create_data


def contract_filmin_create_data(odoo_env, partner):
    vals = contract_create_data(partner)
    vals.update(
        {
            "name": "Test Contract Filmin",
            "service_technology_id": odoo_env.ref(
                "multimedia_somconnexio.service_technology_multimedia"
            ).id,
            "service_supplier_id": odoo_env.ref(
                "filmin_somconnexio.service_supplier_filmin"
            ).id,
        }
    )
    return vals



from odoo.addons.somconnexio.tests.sc_test_case import SCTestCase
from odoo.exceptions import ValidationError


class TestCreateLeadFromPartner(SCTestCase):
    def setUp(self, *args, **kwargs):
        super().setUp(*args, **kwargs)
        self.partner_id = self.browse_ref("somconnexio.res_partner_1_demo")
        self.filmin_product = self.env.ref("filmin_somconnexio.FilminSubscription")
        self.wizard_params = {
            "opportunity": "test",
            "phone_contact": "888888888",
            "product_id": self.filmin_product.id,
            "product_categ_id": self.ref("multimedia_somconnexio.multimedia_service"),
            "bank_id": self.partner_id.bank_ids[0].id,
            "email_id": self.partner_id.id,
        }

    def test_create_lead_with_existing_fiber_service(self):
        """
        Test the manual creation of a multimedia lead for a Filmin product.
        This test checks that a user with previous Filmin services activated
        cannot create another Filmin service petition through this wizard
        """

        self.assertTrue(self.partner_id.has_had_filmin_service)

        wizard = (
            self.env["partner.create.lead.wizard"]
            .with_context(active_id=self.partner_id.id)
            .create(self.wizard_params)
        )

        self.assertRaisesRegex(
            ValidationError,
            "A Filmin contract already exists for this partner: %s."
            % self.partner_id.id,
            wizard.create_lead,
        )

    def test_create_lead_without_existing_fiber_service(self):
        """
        Test the manual creation of a multimedia lead for a Filmin product.
        This test checks that a user without previous Filmin services activated
        can create another Filmin service petition through this wizard
        """

        partner_id = self.env.ref("somconnexio.res_partner_2_demo")
        self.assertFalse(partner_id.has_had_filmin_service)

        wizard = (
            self.env["partner.create.lead.wizard"]
            .with_context(active_id=partner_id.id)
            .create(self.wizard_params)
        )

        crm_lead_action = wizard.create_lead()
        crm_lead = self.env["crm.lead"].browse(crm_lead_action["res_id"])

        self.assertTrue(crm_lead.has_multimedia_lead_lines)



from odoo.addons.somconnexio.tests.sc_test_case import SCTestCase
from odoo.addons.multimedia_somconnexio.tests.helper_service import crm_lead_create
from odoo.exceptions import ValidationError


class CRMLeadLineTest(SCTestCase):
    def setUp(self, *args, **kwargs):
        super().setUp(*args, **kwargs)
        self.partner_id = self.browse_ref("somconnexio.res_partner_2_demo")
        self.filmin_product = self.env.ref("filmin_somconnexio.FilminSubscription")

    def test_create_filmin_contract_with_partner_having_one(self):
        """
        Test that from a filmin crm_lead_line with a partner that does have
        had previous filmin contracts before, no contract can be created,
        raising a ValidationError.
        """
        existing_filmin_contract = self.env.ref("filmin_somconnexio.contract_filmin")
        mm_lead = crm_lead_create(
            self.env, existing_filmin_contract.partner_id, "multimedia"
        )
        mm_crm_lead_line = mm_lead.lead_line_ids[0]
        mm_crm_lead_line.product_id = self.filmin_product

        self.assertRaisesRegex(
            ValidationError,
            "A Filmin contract already exists for this partner: %s."
            % existing_filmin_contract.partner_id.id,
            mm_crm_lead_line.create_multimedia_contract,
        )

    def test_create_filmin_contract_with_partner_first_one(self):
        """
        Test that from a filmin crm_lead_line a contract with the filmin
        service supplier can be created with a partner that does not have
        a filmin contract yet,
        """
        previous_filmin_contracts = self.env["contract.contract"].search(
            [
                ("partner_id", "=", self.partner_id.id),
                (
                    "service_technology_id",
                    "=",
                    self.env.ref(
                        "multimedia_somconnexio.service_technology_multimedia"
                    ).id,
                ),
                (
                    "service_supplier_id",
                    "=",
                    self.env.ref("filmin_somconnexio.service_supplier_filmin").id,
                ),
            ]
        )
        self.assertFalse(previous_filmin_contracts)

        # Create a CRM lead line for Filmin
        mm_lead = crm_lead_create(self.env, self.partner_id, "multimedia")
        mm_crm_lead_line = mm_lead.lead_line_ids[0]
        mm_crm_lead_line.product_id = self.filmin_product

        mm_contract = mm_crm_lead_line.create_multimedia_contract()

        self.assertTrue(mm_contract.id)
        self.assertEqual(
            mm_contract.service_supplier_id,
            self.env.ref("filmin_somconnexio.service_supplier_filmin"),
        )
        self.assertEqual(
            mm_contract.service_technology_id,
            self.env.ref("multimedia_somconnexio.service_technology_multimedia"),
        )
        self.assertEqual(mm_contract.partner_id, self.partner_id)



from odoo.addons.somconnexio.tests.sc_test_case import SCTestCase
from odoo.addons.multimedia_somconnexio.tests.helper_service import crm_lead_create
from mock import patch


class CRMLeadTest(SCTestCase):
    def setUp(self, *args, **kwargs):
        super().setUp(*args, **kwargs)
        self.partner_id = self.browse_ref("somconnexio.res_partner_2_demo")
        self.filmin_product = self.env.ref("filmin_somconnexio.FilminSubscription")
        self.lead = crm_lead_create(
            self.env,
            self.partner_id,
            "multimedia",
        )
        self.lead.lead_line_ids[0].product_id = self.filmin_product
        self.fiber_lead = crm_lead_create(
            self.env,
            self.partner_id,
            "fiber",
        )

    def test_crm_lead_has_single_filmin_line(self):
        """Test that a CRM lead with a single Filmin line is correctly identified."""

        self.assertTrue(len(self.lead.lead_line_ids) == 1)
        self.assertEqual(self.lead.lead_line_ids[0].product_id, self.filmin_product)
        self.lead._compute_has_single_filmin_line()
        self.assertTrue(self.lead.has_single_filmin_line)

    def test_crm_lead_has_not_single_filmin_line(self):
        """Test that a CRM lead mixed with Filmin is not identified
        as having a single Filmin line."""
        self.lead.lead_line_ids |= self.fiber_lead.lead_line_ids[0]

        self.assertTrue(len(self.lead.lead_line_ids) == 2)
        self.lead._compute_has_single_filmin_line()
        self.assertEqual(self.lead.lead_line_ids[0].product_id, self.filmin_product)
        self.assertNotEqual(self.lead.lead_line_ids[1].product_id, self.filmin_product)
        self.assertFalse(self.lead.has_single_filmin_line)

    @patch("odoo.addons.somconnexio.models.crm_lead.CrmLead.action_send_email")
    def test_action_send_mail_single_filmin_line(self, mock_action_send_email):
        """Test that the action_send_email does not send an email
        if there is a single Filmin line."""
        self.assertTrue(self.lead.has_single_filmin_line)
        self.lead.action_send_email()
        mock_action_send_email.assert_not_called()

    @patch("odoo.addons.somconnexio.models.crm_lead.CrmLead.action_send_email")
    def test_action_send_mail_fiber_line(self, mock_action_send_email):
        """Test that the action_send_email sends an email
        if there is a fiber line."""
        self.assertFalse(self.fiber_lead.has_single_filmin_line)
        self.fiber_lead.action_send_email()
        mock_action_send_email.assert_called_once()



from odoo import models


class CRMLeadAddMultimediaLine(models.TransientModel):
    _inherit = "crm.lead.add.multimedia.line.wizard"

    def button_create(self):
        self.ensure_one()

        filmin_product = self.env.ref("filmin_somconnexio.FilminSubscription")

        if self.product_id == filmin_product:
            self.partner_id.check_filmin_service_allowed()

        return super().button_create()



from odoo.addons.somconnexio.tests.sc_test_case import SCTestCase


class TestContract(SCTestCase):
    def setUp(self, *args, **kwargs):
        super().setUp(*args, **kwargs)
        self.contract = self.env.ref("filmin_somconnexio.contract_filmin")

    def test_is_filmin(self):
        self.assertTrue(self.contract.is_filmin)

    def test_is_not_filmin(self):
        contract = self.env.ref("somconnexio.contract_mobile_il_20")
        self.assertFalse(contract.is_filmin)

    def test_contract_name(self):
        self.assertEqual(
            self.contract.name, f"FILMIN - {self.contract.subscription_code}"
        )

    def test_contract_type(self):
        self.assertEqual(self.contract.service_contract_type, "filmin")

    def test_contract_type_not_filmin(self):
        contract = self.env.ref("somconnexio.contract_mobile_il_20")
        self.assertNotEqual(contract.service_contract_type, "filmin")

    def test_filmin_contract_to_dict(self):
        self.assertEqual(self.contract._to_dict()["subscription_technology"], "filmin")



from odoo import fields, models, _
from odoo.exceptions import ValidationError


class ResPartner(models.Model):
    _inherit = "res.partner"

    has_had_filmin_service = fields.Boolean(
        string="Has had Filmin service",
        compute="_compute_has_had_filmin_service",
        defaults=False,
    )

    def _compute_has_had_filmin_service(self):
        """
        Compute if the partner has had a Filmin contract.
        This is used to determine if the partner can add a new Filmin line.
        """
        self.env.ref("filmin_somconnexio.FilminSubscription")
        multimedia_technology = self.env.ref(
            "multimedia_somconnexio.service_technology_multimedia"
        )

        for partner in self:
            previous_partner_filmin_contracts = self.env["contract.contract"].search(
                [
                    ("partner_id", "=", partner.id),
                    ("is_filmin", "=", True),
                    (
                        "service_technology_id",
                        "=",
                        multimedia_technology.id,
                    ),
                ]
            )
            partner.has_had_filmin_service = bool(previous_partner_filmin_contracts)

    def check_filmin_service_allowed(self):
        self.ensure_one()
        self.env.ref("filmin_somconnexio.FilminSubscription")
        if self.has_had_filmin_service:
            raise ValidationError(
                _("A Filmin contract already exists for this partner: %s.") % self.id
            )



from odoo.addons.somconnexio.tests.sc_test_case import SCTestCase
from odoo.addons.multimedia_somconnexio.tests.helper_service import crm_lead_create
from mock import patch


class IrServerActionTest(SCTestCase):
    def setUp(self, *args, **kwargs):
        super().setUp(*args, **kwargs)
        self.partner_id = self.browse_ref("somconnexio.res_partner_2_demo")
        self.filmin_product = self.env.ref("filmin_somconnexio.FilminSubscription")
        self.lead = crm_lead_create(
            self.env,
            self.partner_id,
            "multimedia",
        )
        self.lead.lead_line_ids[0].product_id = self.filmin_product
        self.fiber_lead = crm_lead_create(
            self.env,
            self.partner_id,
            "fiber",
        )
        self.IrActionsServer = self.env["ir.actions.server"]
        self.action = object()

    @patch(
        "odoo.addons.somconnexio.models.ir_server_action.ServerActions.run_action_background_email"  # noqa: E501
    )
    def test_action_run_background_email_single_filmin_line(
        self, mock_run_action_background_email
    ):
        """Test that the run_action_background_email does not send an email
        if there is a single Filmin line."""
        self.assertTrue(self.lead.has_single_filmin_line)

        self.IrActionsServer.with_context(
            active_id=self.lead.id
        ).run_action_background_email(self.action)
        mock_run_action_background_email.assert_not_called()

    @patch(
        "odoo.addons.somconnexio.models.ir_server_action.ServerActions.run_action_background_email"  # noqa: E501
    )
    def test_action_run_background_email_fiber_line(
        self, mock_run_action_background_email
    ):
        """Test that the run_action_background_email sends an email
        if there is a fiber line."""
        self.assertFalse(self.fiber_lead.has_single_filmin_line)
        self.IrActionsServer.with_context(
            active_id=self.fiber_lead.id
        ).run_action_background_email(self.action)
        mock_run_action_background_email.assert_called_once_with(self.action, None)



from odoo import models, fields


class CRMLead(models.Model):
    _inherit = "crm.lead"

    has_single_filmin_line = fields.Boolean(
        string="Is only one Filmin line",
        help="Indicates if this lead has only one Filmin line.",
        compute="_compute_has_single_filmin_line",
        default=False,
    )

    def _compute_has_single_filmin_line(self):
        for lead in self:
            lead.has_single_filmin_line = len(
                lead.lead_line_ids
            ) == 1 and lead.lead_line_ids[0].product_id == self.env.ref(
                "filmin_somconnexio.FilminSubscription"
            )

    def action_send_email(self):
        """Override to ensure the email is not sent if there is a single Filmin line."""
        for lead in self:
            if lead.has_single_filmin_line:
                return False
            else:
                return super(CRMLead, lead).action_send_email()



from odoo import models


class CRMLeadLine(models.Model):
    _inherit = "crm.lead.line"

    def create_multimedia_contract(self):
        """
        Check that no previous filmin contracts exist for the lead partner,
        and create a new multimedia contract from the lead line.
        If the product is Filmin, set the service supplier to Filmin.
        """

        filmin_product = self.env.ref("filmin_somconnexio.FilminSubscription")
        if self.product_id == filmin_product:
            partner_id = self.lead_id.partner_id
            partner_id.check_filmin_service_allowed()

        return super().create_multimedia_contract()

    def _get_service_supplier(self):
        """
        Get the filmin service supplier contract.
        This overrides the unimplemented method in multimedia_somconnexio
        :return: Service supplier record
        """
        if self.product_id == self.env.ref("filmin_somconnexio.FilminSubscription"):
            return self.env.ref("filmin_somconnexio.service_supplier_filmin")



from odoo import models, fields, api


class ContractLine(models.Model):
    _inherit = "contract.line"

    lot_id = fields.Many2one("stock.lot", string="Serial Number")

    @api.model
    def create(self, vals):
        """
        Override the create method to handle the creation of a contract line
        and automatically assign a serial number if the product is tracked by
        serial number.
        Also, create a stock move for the internal consumption of the product.
        """
        res = super().create(vals)

        if res.lot_id:
            return res

        product = res.product_id
        if (
            product
            and product.detailed_type == "product"
            and product.tracking == "serial"
        ):
            lot = self._pick_lot_from_stock(res, product)
            self._assign_lot_to_contract(res, lot)

        return res

    def _assign_lot_to_contract(self, res, lot):
        """
        Assign the lot to the contract line and
        the lot name as the contract's subscription code.
        """
        res.lot_id = lot
        res.contract_id.subscription_code = lot.name

    def _pick_lot_from_stock(self, res, product):
        """
        Create a stock picking to move a unit out of stock
        """
        picking = self.env["stock.picking"].create(
            {
                "picking_type_id": self.env.ref("stock.picking_type_out").id,
                "location_id": self.env.ref("stock.stock_location_stock").id,
                "location_dest_id": self.env.ref("stock.stock_location_customers").id,
                "origin": f"Pick out {product.showed_name} for contract {res.contract_id.id}",  # noqa: E501
            }
        )

        move = self.env["stock.move"].create(
            {
                "name": f"Move out {product.showed_name} for contract {res.contract_id.id}",  # noqa: E501
                "picking_id": picking.id,
                "product_id": product.id,
                "product_uom_qty": 1.0,
                "product_uom": product.uom_id.id,
                "location_id": picking.location_id.id,
                "location_dest_id": picking.location_dest_id.id,
            }
        )
        move._action_confirm()
        move._action_assign()

        # One move line generated per move with serial product
        move_line = move.move_line_ids[0]
        move_line.write({"qty_done": 1.0})

        picking.action_confirm()
        picking.action_assign()
        picking.button_validate()

        return move_line.lot_id



from datetime import date
from odoo.addons.somconnexio.tests.sc_test_case import SCTestCase
from ..helper_service import contract_filmin_create_data


class TestContractLine(SCTestCase):
    def setUp(self):
        super().setUp()
        self.ContractLine = self.env["contract.line"]
        self.Quant = self.env["stock.quant"]
        self.stock_location = self.env.ref("stock.stock_location_stock")
        self.partner = self.env.ref("somconnexio.res_partner_company_demo")
        self.serial_product = self.env.ref("filmin_somconnexio.FilminSubscription")

        self.new_contract = self.env["contract.contract"].create(
            contract_filmin_create_data(self.env, self.partner)
        )

    def test_contract_line_creation_with_serial_product(self):
        """
        Test that if a contract line is created with a serial tracking product
        a stock lot is picked out from supply and assigned to the contract line
        and the contract's subscription code is set to the lot name.
        """
        self.assertEqual(self.serial_product.tracking, "serial")
        self.assertEqual(self.serial_product.detailed_type, "product")

        stock_quant_before = self.Quant.search(
            [
                ("product_id", "=", self.serial_product.id),
                ("location_id", "=", self.stock_location.id),
                ("quantity", "=", 1),
            ]
        )

        self.assertTrue(stock_quant_before)

        # create a new contract line with serial tracked product
        line = self.ContractLine.create(
            {
                "name": "Test Line",
                "contract_id": self.new_contract.id,
                "product_id": self.serial_product.id,
                "date_start": date.today(),
            }
        )

        stock_quant_after = self.Quant.search(
            [
                ("product_id", "=", self.serial_product.id),
                ("location_id", "=", self.stock_location.id),
                ("quantity", "=", 1),
            ]
        )

        lot_used = (stock_quant_before - stock_quant_after).lot_id
        self.assertTrue(lot_used)
        self.assertTrue(len(lot_used) == 1)
        self.assertEqual(line.lot_id, lot_used)
        self.assertEqual(line.contract_id.subscription_code, lot_used.name)

        move_out = self.env["stock.move"].search(
            [
                (
                    "name",
                    "=",
                    f"Move out {self.serial_product.showed_name} for contract {line.contract_id.id}",  # noqa: E501
                ),
                ("state", "=", "done"),
            ],
            limit=1,
        )

        self.assertTrue(move_out)
        self.assertEqual(move_out.product_id, self.serial_product)
        self.assertEqual(move_out.product_uom_qty, 1.0)
        self.assertEqual(move_out.location_id, self.stock_location)
        self.assertEqual(
            move_out.location_dest_id, self.env.ref("stock.stock_location_customers")
        )
        self.assertEqual(
            move_out.picking_type_id, self.env.ref("stock.picking_type_out")
        )
        self.assertEqual(move_out.move_line_ids[0].lot_id, lot_used)
        self.assertEqual(move_out.move_line_ids[0].qty_done, 1.0)
        self.assertTrue(move_out.picking_id)
        self.assertEqual(move_out.picking_id.state, "done")

    def test_non_serial_product_does_not_assign_lot(self):
        """
        Test that if a contract line is created with a non serial tracking product
        no stock lot is picked out from supply and none is assigned to the contract line
        """
        non_serial_product = self.serial_product.copy({"tracking": "none"})
        line = self.ContractLine.create(
            {
                "name": "Non-serial Line",
                "contract_id": self.new_contract.id,
                "product_id": non_serial_product.id,
                "date_start": date.today(),
            }
        )

        move_out = self.env["stock.move"].search(
            [
                (
                    "name",
                    "=",
                    f"Move out {non_serial_product.showed_name} for contract {line.contract_id.id}",  # noqa: E501
                ),
                ("state", "=", "done"),
            ],
            limit=1,
        )
        self.assertFalse(move_out)
        self.assertFalse(line.lot_id)
        self.assertFalse(line.contract_id.subscription_code)



from . import models, listeners, wizards
from odoo import api, SUPERUSER_ID
import logging

_logger = logging.getLogger(__name__)


def _demo_stock_lots_move(cr, registry):
    """
    Post init hook to create a stock move for the demo data
    """

    env = api.Environment(cr, SUPERUSER_ID, {})

    is_dev = _is_this_dev(env)
    if not is_dev:
        _logger.info("❌ Skipping Filmin demo hook, not in development mode.")
        return

    product = env.ref("filmin_somconnexio.FilminSubscription")
    lots = [
        env.ref("filmin_somconnexio.stock_filmin_01"),
        env.ref("filmin_somconnexio.stock_filmin_02"),
        env.ref("filmin_somconnexio.stock_filmin_03"),
    ]

    # ---------------------
    # Move lots IN as stock
    # ---------------------

    move_in = env["stock.move"].create(
        {
            "name": "Bring serial Filmin products in stock",
            "location_id": env.ref("stock.stock_location_suppliers").id,
            "location_dest_id": env.ref("stock.stock_location_stock").id,
            "product_id": product.id,
            "product_uom": product.uom_id.id,
            "product_uom_qty": float(len(lots)),
        }
    )
    move_in._action_confirm()

    # Assing the 3 demo stock.lots with stock.move.lines
    move_in.move_line_ids.unlink()
    for lot in lots:
        env["stock.move.line"].create(
            {
                "move_id": move_in.id,
                "location_id": move_in.location_id.id,
                "location_dest_id": move_in.location_dest_id.id,
                "product_id": product.id,
                "product_uom_id": product.uom_id.id,
                "qty_done": 1.0,
                "lot_id": lot.id,
                "lot_name": lot.name,
            }
        )
    move_in._action_done()

    _logger.info("✅ Demo move from Filmin to stock correctly processed.")

    # ------------------------------
    # Move lot FLM0003 out from stock
    # -------------------------------

    move_out = env["stock.move"].create(
        {
            "name": "Send one serial Filmin product out",
            "location_id": env.ref("stock.stock_location_stock").id,
            "location_dest_id": env.ref("stock.stock_location_customers").id,
            "product_id": product.id,
            "product_uom": product.uom_id.id,
            "product_uom_qty": 1.0,
        }
    )
    move_out._action_confirm()
    move_out._action_assign()

    # Assing the FLM0003 demo stock.lots to stock.move.line (generated automatically)
    lot_used = lots[-1]
    move_out_line = move_out.move_line_ids[0]
    move_out_line.write(
        {
            "lot_id": lot_used.id,
            "lot_name": lot_used.name,
            "qty_done": 1.0,
        }
    )
    move_out._action_done()

    _logger.info("✅ Demo move out from stock correctly processed.")


def _is_this_dev(env):
    """
    Check if the current environment is in development mode.
    This is used to avoid running demo hooks in production environments.
    """
    demo_lot = env.ref("filmin_somconnexio.stock_filmin_01", raise_if_not_found=False)
    return bool(demo_lot and demo_lot.exists())



from .models import (
    test_contract,
    test_contract_line,
    test_crm_lead_line,
    test_crm_lead,
    test_ir_server_action,
    test_res_partner,
)
from .listeners import (
    test_crm_lead_listener,
)
from .wizards import test_create_lead_from_partner, test_crm_lead_add_multimedia_line
from . import helper_service



from odoo.addons.somconnexio.tests.sc_test_case import SCTestCase
from odoo.addons.multimedia_somconnexio.tests.helper_service import crm_lead_create
from odoo.exceptions import ValidationError


class TestCRMLeadAddMultimediaLine(SCTestCase):
    def setUp(self, *args, **kwargs):
        super().setUp(*args, **kwargs)
        self.partner_id = self.env.ref("somconnexio.res_partner_1_demo")
        self.filmin_product = self.env.ref("filmin_somconnexio.FilminSubscription")

    def test_create_line_with_existing_fiber_service(self):
        """
        Test the creation of a multimedia line in a CRM lead for a Filmin product.
        This test checks that a user with previous Filmin services activated
        cannot add another Filmin service through this wizard
        """

        self.assertTrue(self.partner_id.has_had_filmin_service)

        fiber_crm_lead = crm_lead_create(
            self.env, self.partner_id, "fiber", portability=False
        )

        self.assertFalse(fiber_crm_lead.has_multimedia_lead_lines)

        wizard_vals = {
            "product_id": self.filmin_product.id,
            "bank_id": self.partner_id.bank_ids.id,
        }
        wizard = (
            self.env["crm.lead.add.multimedia.line.wizard"]
            .with_context(active_id=fiber_crm_lead.id)
            .create(wizard_vals)
        )

        self.assertRaisesRegex(
            ValidationError,
            "A Filmin contract already exists for this partner: %s."
            % self.partner_id.id,
            wizard.button_create,
        )

    def test_create_line_with_no_existing_fiber_service(self):
        """
        Test the creation of a multimedia line in a CRM lead for a Filmin product.
        This test checks that a user without previous Filmin services activated
        can add another Filmin service through this wizard
        """
        partner_id = self.env.ref("somconnexio.res_partner_2_demo")
        self.assertFalse(partner_id.has_had_filmin_service)

        fiber_crm_lead = crm_lead_create(
            self.env, partner_id, "fiber", portability=False
        )

        self.assertFalse(fiber_crm_lead.has_multimedia_lead_lines)

        wizard_vals = {
            "product_id": self.filmin_product.id,
            "bank_id": partner_id.bank_ids.id,
        }
        wizard = (
            self.env["crm.lead.add.multimedia.line.wizard"]
            .with_context(active_id=fiber_crm_lead.id)
            .create(wizard_vals)
        )
        wizard.button_create()

        self.assertTrue(fiber_crm_lead.has_multimedia_lead_lines)



from odoo.addons.somconnexio.tests.sc_test_case import SCTestCase
from odoo.exceptions import ValidationError


class ResPartnerTest(SCTestCase):
    def setUp(self, *args, **kwargs):
        super().setUp(*args, **kwargs)
        self.partner_id = self.browse_ref("somconnexio.res_partner_1_demo")

    def test_has_had_filmin_service(self):
        """Test that a partner with a Filmin contract has the
        has_had_filmin_service field set to True."""

        previous_filmin_contracts = self.env["contract.contract"].search(
            [
                ("partner_id", "=", self.partner_id.id),
                (
                    "service_technology_id",
                    "=",
                    self.env.ref(
                        "multimedia_somconnexio.service_technology_multimedia"
                    ).id,
                ),
                (
                    "is_filmin",
                    "=",
                    True,
                ),
            ]
        )
        self.assertTrue(previous_filmin_contracts)
        self.assertTrue(self.partner_id.has_had_filmin_service)

    def test_check_filmin_service_allowed(self):
        """Test that a partner with a Filmin contract raises an error
        when trying to create another Filmin service."""

        self.assertTrue(self.partner_id.has_had_filmin_service)

        self.assertRaisesRegex(
            ValidationError,
            "A Filmin contract already exists for this partner: %s."
            % self.partner_id.id,
            self.partner_id.check_filmin_service_allowed,
        )

    def test_has_not_had_filmin_service(self):
        """Test that a partner without a Filmin contract has the
        has_had_filmin_service field set to False."""

        partner_id = self.env.ref("somconnexio.res_partner_2_demo")
        previous_filmin_contracts = self.env["contract.contract"].search(
            [
                ("partner_id", "=", partner_id.id),
                (
                    "service_technology_id",
                    "=",
                    self.env.ref(
                        "multimedia_somconnexio.service_technology_multimedia"
                    ).id,
                ),
                (
                    "is_filmin",
                    "=",
                    True,
                ),
            ]
        )
        self.assertFalse(previous_filmin_contracts)
        self.assertFalse(partner_id.has_had_filmin_service)

    def test_check_filmin_service_allowed_no_previous_service(self):
        """Test that a partner without a Filmin contract does not raise an error
        when trying to create a Filmin service."""

        partner_id = self.env.ref("somconnexio.res_partner_2_demo")
        self.assertFalse(partner_id.has_had_filmin_service)

        # This should not raise an error
        partner_id.check_filmin_service_allowed()



import logging

from odoo import models

_log = logging.getLogger(__name__)


class ServerActions(models.Model):
    """Add email option in server actions."""

    _name = "ir.actions.server"
    _description = "Server Action"
    _inherit = ["ir.actions.server"]

    def run_action_background_email(self, action, eval_context=None):
        active_id = self.env.context["active_id"]
        # Do not send mails for single Filmin line
        crm_lead = self.env["crm.lead"].browse(active_id)
        if crm_lead.has_single_filmin_line:
            return
        else:
            super(ServerActions, self).run_action_background_email(action, eval_context)



from odoo.addons.component.core import Component


class CrmLeadListener(Component):
    _name = "filmin.crm.lead.listener"
    _inherit = "multimedia.crm.lead.listener"

    def on_record_create(self, record, fields=None):
        """
        This method is triggered when a CRM lead record is created.
        It checks if the lead has just one lead line with filmin supplier.
        If so, it automatically validates the lead and sets it to the 'won' stage.
        """
        if record.has_single_filmin_line:
            record.action_set_remesa()
            record.action_set_won()



from odoo.addons.somconnexio.tests.sc_test_case import SCComponentTestCase
from odoo.addons.multimedia_somconnexio.tests.helper_service import crm_lead_create


class TestCRMLeadListener(SCComponentTestCase):
    @classmethod
    def setUpClass(cls):
        super(TestCRMLeadListener, cls).setUpClass()
        # disable tracking test suite wise
        cls.env = cls.env(
            context=dict(
                cls.env.context,
                tracking_disable=True,
                test_queue_job_no_delay=False,
            )
        )

    def setUp(self, *args, **kwargs):
        super().setUp(*args, **kwargs)
        self.partner_id = self.browse_ref("somconnexio.res_partner_2_demo")
        self.filmin_product = self.env.ref("filmin_somconnexio.FilminSubscription")
        self.crm_lead_line_filmin_vals = {
            "name": "Filmin Lead Line",
            "iban": self.partner_id.bank_ids[0].sanitized_acc_number,
            "product_id": self.filmin_product.id,
        }
        self.crm_lead_filmin_vals = {
            "name": "Test Lead",
            "partner_id": self.partner_id.id,
            "phone": self.partner_id.phone,
            "email_from": self.partner_id.email,
            "stage_id": self.env.ref("crm.stage_lead1").id,
            "lead_line_ids": [
                (
                    0,
                    0,
                    self.crm_lead_line_filmin_vals,
                )
            ],
        }

    def test_crm_lead_lead_single_filmin_validated_at_creation(self):
        """
        Test that a CRM lead with a single Filmin line is not validated
        at creation.
        """
        self.assertEqual(
            self.crm_lead_filmin_vals["stage_id"],
            self.env.ref("crm.stage_lead1").id,  # Draft
        )
        filmin_lead = self.env["crm.lead"].create(self.crm_lead_filmin_vals)
        self.assertTrue(
            filmin_lead.has_single_filmin_line,
        )
        self.assertEqual(
            filmin_lead.stage_id,
            self.env.ref("crm.stage_lead4"),  # Won
        )

    def test_crm_lead_lead_not_single_filmin_not_validated_at_creation(self):
        """
        Test that a CRM lead with False has_single_filmin_line is not validated
        at creation.
        """
        fiber_lead = crm_lead_create(
            self.env,
            self.partner_id,
            "fiber",
        )
        self.assertFalse(
            fiber_lead.has_single_filmin_line,
        )
        self.assertEqual(
            fiber_lead.stage_id,
            self.env.ref("crm.stage_lead1"),  # Draft
        )



from odoo import api, fields, models


class Contract(models.Model):
    _inherit = "contract.contract"

    is_filmin = fields.Boolean(
        string="Is FILMIN",
        compute="_compute_is_filmin",
        store=True,
    )

    @api.depends("service_supplier_id")
    def _compute_is_filmin(self):
        filmin_supplier = self.env.ref(
            "filmin_somconnexio.service_supplier_filmin", raise_if_not_found=False
        )
        if not filmin_supplier:
            return
        for record in self:
            record.is_filmin = record.service_supplier_id == filmin_supplier

    @api.depends("subscription_code")
    def _compute_name(self):
        super(Contract, self)._compute_name()
        for contract in self:
            if contract.is_filmin:
                contract.name = f"FILMIN - {contract.subscription_code}"

    @api.depends("service_technology_id")
    def _compute_contract_type(self):
        super(Contract, self)._compute_contract_type()
        for record in self:
            if record.is_filmin:
                record.service_contract_type = "filmin"

    def _get_subscription_tech(self):
        """overrides method that inform subscription_technology in _to_dict parent"""
        return (
            self.service_contract_type
            if self.is_filmin
            else super()._get_subscription_tech()
        )



{
    "name": "filmin_somconnexio",
    "version": "16.0.1.0.3",
    "depends": ["multimedia_somconnexio"],
    "summary": """
        Manages filmin subscriptions within the SomConnexio ERP
    """,
    "author": "Coopdevs Treball SCCL, " "Som Connexió SCCL",
    "website": "https://coopdevs.org",
    "license": "AGPL-3",
    "category": "Cooperative management",
    "data": [
        "data/service_supplier.xml",
        "data/sale_subscription_template.xml",
        "data/service_technology_service_supplier.xml",
        "data/product_category_technology_supplier_data.xml",
        "data/product_product.xml",
        "views/contract_views.xml",
        "reports/crm_lead_creation_email_template.xml",
        "reports/crm_lead_creation_manual_email_template.xml",
    ],
    "demo": [
        "demo/stock_lot.xml",
        "demo/contract.xml",
    ],
    "post_init_hook": "_demo_stock_lots_move",
}



from odoo import models


class CreateLeadFromPartner(models.TransientModel):
    _inherit = "partner.create.lead.wizard"

    def create_lead(self):
        self.ensure_one()
        filmin_product = self.env.ref("filmin_somconnexio.FilminSubscription")

        if self.product_id == filmin_product:
            self.partner_id.check_filmin_service_allowed()

        return super().create_lead()
